@@ -17,11 +17,11 @@ from typing import TypeVar, Iterable
 from xdsl.utils.exceptions import ParseError, MultipleSpansParseError
 from xdsl.dialects.memref import MemRefType, UnrankedMemrefType
 from xdsl.dialects.builtin import (
-    AnyTensorType, AnyVectorType, Float16Type, Float32Type, Float64Type,
-    FloatAttr, FunctionType, IndexType, IntegerType, Signedness, StringAttr,
-    IntegerAttr, ArrayAttr, TensorType, UnrankedTensorType, VectorType,
-    FlatSymbolRefAttr, DenseIntOrFPElementsAttr, UnregisteredOp, OpaqueAttr,
-    NoneAttr, ModuleOp, UnitAttr, i64)
+    AnyTensorType, AnyVectorType, DenseResourceAttr, Float16Type, Float32Type,
+    Float64Type, FloatAttr, FunctionType, IndexType, IntegerType, Signedness,
+    StringAttr, IntegerAttr, ArrayAttr, TensorType, UnrankedTensorType,
+    VectorType, FlatSymbolRefAttr, DenseIntOrFPElementsAttr, UnregisteredOp,
+    OpaqueAttr, NoneAttr, ModuleOp, UnitAttr, i64)
 from xdsl.ir import (SSAValue, Block, Callable, Attribute, Operation, Region,
                      BlockArgument, MLContext, ParametrizedAttribute, Data)
 
@@ -666,10 +666,14 @@ class BaseParser(ABC):
             block = Block(block_id)
             self.blocks[block_id.text] = block
 
+        block_args: list[BlockArgument] = []
+
         for i, (name, type) in enumerate(args):
             arg = BlockArgument(type, block, i)
             self.ssaValues[name.text] = arg
-            block.args.append(arg)
+            block_args.append(arg)
+
+        block._args = tuple(block_args)  # type: ignore
 
         while (next_op := self.try_parse_operation()) is not None:
             block.add_op(next_op)
@@ -786,6 +790,17 @@ class BaseParser(ABC):
     def try_parse_value_id(self) -> Span | None:
         return self.tokenizer.next_token_of_pattern(ParserCommons.value_id)
 
+    def try_parse_operand(self) -> SSAValue | None:
+        """Try to parse an operand with format `%<value-id>`."""
+        value_id = self.try_parse_value_id()
+        if value_id is None:
+            return None
+        return self.get_ssa_val(value_id)
+
+    def parse_operand(self, msg: str = "operand expected") -> SSAValue:
+        """Parse an operand with format `%<value-id>`."""
+        return self.expect(self.try_parse_operand, msg)
+
     def try_parse_suffix_id(self) -> Span | None:
         return self.tokenizer.next_token_of_pattern(ParserCommons.suffix_id)
 
@@ -1204,7 +1219,8 @@ class BaseParser(ABC):
         """
         Parse attribute (either builtin or dialect)
 
-        This is different in xDSL and MLIR, so the actuall implementation is provided by the subclass
+        This is different in xDSL and MLIR, so the actuall implementation is
+        provided by the subclass
         """
         raise NotImplementedError()
 
@@ -1255,6 +1271,7 @@ class BaseParser(ABC):
             parsers = {
                 'dense': self._parse_builtin_dense_attr,
                 'opaque': self._parse_builtin_opaque_attr,
+                'dense_resource': self._parse_builtin_dense_resource_attr,
             }
 
             def not_implemented():
@@ -1293,6 +1310,17 @@ class BaseParser(ABC):
                                          for span in str_lit_list),
                                        type=type)
 
+    def _parse_builtin_dense_resource_attr(self) -> DenseResourceAttr:
+        err_msg = ("Malformed dense_resource attribute, format must be "
+                   "(`dense_resource` `<` resource-handle `>`)")
+        self.parse_characters("<", err_msg)
+        resource_handle = self.expect(self.try_parse_bare_id, err_msg)
+        self.parse_characters(">", err_msg)
+        self.parse_characters(":", err_msg)
+        type = self.expect(self.try_parse_type,
+                           "Dense resource attribute must be typed!")
+        return DenseResourceAttr.from_params(resource_handle.text, type)
+
     def _parse_builtin_dense_attr_args(self) -> Iterable[int | float]:
         """
         Dense attribute params must be:
@@ -1396,7 +1424,6 @@ class BaseParser(ABC):
                                          Attribute]]) -> dict[str, Attribute]:
         """
         Convert a list of tuples (Span, Attribute) to a dictionary.
-
         This function converts the span to a string, trimming quotes from string literals
         """
 
@@ -1447,8 +1474,8 @@ class BaseParser(ABC):
         type-list-no-parens      ::=  type (`,` type)*
         """
         if self.tokenizer.next_token_of_pattern("(") is not None:
-            args: list[Attribute] = self.parse_list_of(self.try_parse_type,
-                                                       "Expected type here!")
+            args: list[Attribute | None] = self.parse_list_of(
+                self.try_parse_type, "Expected type here!")
             self.parse_characters(")", "Unclosed function type argument list!")
         else:
             args = [self.try_parse_type()]
@@ -1513,8 +1540,8 @@ class BaseParser(ABC):
             - a list of successor names
             - the attributes attached to the OP
             - the regions of the op
-            - An optional function type. If not supplied, parse_op_result_list must return a second value
-              containing the types of the returned SSAValues
+            - An optional function type. If not supplied, `parse_op_result_list`
+              must return a second value containing the types of the returned SSAValues
 
         """
         raise NotImplementedError()
@@ -1524,8 +1551,9 @@ class BaseParser(ABC):
         raise NotImplementedError()
 
     # HERE STARTS A SOMEWHAT CURSED COMPATIBILITY LAYER:
-    # Since we don't want to rewrite all dialects currently, the new parser needs to expose the same
-    # Interface to the dialect definitions (to some extent). Here we implement that interface.
+    # Since we don't want to rewrite all dialects currently, the new parser needs
+    # to expose the same Interface to the dialect definitions (to some extent).
+    # Here we implement that interface.
 
     _OperationType = TypeVar("_OperationType", bound=Operation)
 
@@ -1535,8 +1563,8 @@ class BaseParser(ABC):
         result_types: list[Attribute],
     ) -> _OperationType:
         """
-        Compatibility wrapper so the new parser can be passed instead of the old one. Parses everything after the
-        operation name.
+        Compatibility wrapper so the new parser can be passed instead of the old one.
+        Parses everything after the operation name.
 
         This implicitly assumes XDSL format, and will fail on MLIR style operations
         """
@@ -1637,6 +1665,12 @@ class MLIRParser(BaseParser):
 
             return value
 
+        # In MLIR, a type can be parsed at any attribute location.
+        # While MLIR wraps the type in a `TypeAttr`, we do not require this
+        # in xDSL.
+        if (type := self.try_parse_type()) is not None:
+            return type
+
         # If it isn't a dialect attr, parse builtin
         builtin_val = self.try_parse_builtin_attr()
 
