@@ -3,7 +3,7 @@ from __future__ import annotations
 import inspect
 from abc import ABC, abstractmethod
 from dataclasses import dataclass, field
-from typing import Any, Callable
+from typing import Callable, TypeVar
 
 from xdsl.dialects.builtin import ModuleOp
 from xdsl.ir import (Operation, Region, Block, BlockArgument, Attribute,
@@ -318,8 +318,11 @@ class RewritePattern(ABC):
     A side-effect free rewrite pattern matching on a DAG.
     """
 
+    # The / in the function signature makes the previous arguments positional, see
+    # https://peps.python.org/pep-0570/
+    # This is used by the op_type_rewrite_pattern
     @abstractmethod
-    def match_and_rewrite(self, op: Operation, rewriter: PatternRewriter):
+    def match_and_rewrite(self, op: Operation, rewriter: PatternRewriter, /):
         """
         Match an operation, and optionally perform a rewrite using the rewriter.
         """
@@ -331,16 +334,38 @@ class AnonymousRewritePattern(RewritePattern):
     """
     A rewrite pattern encoded by an anonymous function.
     """
-    func: Callable[[Operation, PatternRewriter], None]
+    func: Callable[[RewritePattern, Operation, PatternRewriter], None]
+
+    def __init__(
+        self,
+        func: Callable[[RewritePattern, Operation, PatternRewriter], None]
+        | Callable[[Operation, PatternRewriter], None]):
+        params = [
+            param for param in inspect.signature(func).parameters.values()
+        ]
+        if len(params) == 2:
+
+            def new_func(self: RewritePattern, op: Operation,
+                         rewriter: PatternRewriter):
+                func(op, rewriter)  # type: ignore
+
+            self.func = new_func
+        else:
+            self.func = func  # type: ignore
 
     def match_and_rewrite(self, op: Operation,
                           rewriter: PatternRewriter) -> None:
-        self.func(op, rewriter)
+        self.func(self, op, rewriter)
+
+
+_RewritePatternT = TypeVar("_RewritePatternT", bound=RewritePattern)
+_OperationT = TypeVar("_OperationT", bound=Operation)
 
 
 def op_type_rewrite_pattern(
-    func: Callable[..., None]
-) -> Callable[[Any, Any], None] | Callable[[Any, Any, Any], None]:
+    func: Callable[[_RewritePatternT, _OperationT, PatternRewriter], None]
+    | Callable[[_OperationT, PatternRewriter], None]
+) -> Callable[[_RewritePatternT, Operation, PatternRewriter], None]:
     """
     This function is intended to be used as a decorator on a RewritePatter method.
     It uses type hints to match on a specific operation type before calling the decorated
@@ -363,7 +388,7 @@ def op_type_rewrite_pattern(
             raise Exception(
                 "op_type_rewrite_pattern expects the decorated function to "
                 "have two arguments.")
-    expected_type = params[-2].annotation
+    expected_type: type[_OperationT] = params[-2].annotation
     if not issubclass(expected_type, Operation):
         raise Exception(
             "op_type_rewrite_pattern expects the first non-self argument"
@@ -372,22 +397,22 @@ def op_type_rewrite_pattern(
     if not is_method:
 
         def op_type_rewrite_pattern_static_wrapper(
-                op: Operation, rewriter: PatternRewriter) -> None:
+                self: RewritePattern, op: Operation,
+                rewriter: PatternRewriter) -> None:
             if not isinstance(op, expected_type):
                 return None
-            func(op, rewriter)
+            func(op, rewriter)  # type: ignore
 
         return op_type_rewrite_pattern_static_wrapper
 
     def op_type_rewrite_pattern_method_wrapper(
-            self,  # type: ignore
-            op: Operation,
+            self: _RewritePatternT, op: Operation,
             rewriter: PatternRewriter) -> None:
         if not isinstance(op, expected_type):
             return None
-        func(self, op, rewriter)
+        func(self, op, rewriter)  # type: ignore
 
-    return op_type_rewrite_pattern_method_wrapper  # type: ignore
+    return op_type_rewrite_pattern_method_wrapper
 
 
 @dataclass(eq=False, repr=False)
