@@ -3,7 +3,6 @@ from __future__ import annotations
 import json
 from dataclasses import dataclass, field
 from enum import Enum
-from frozenlist import FrozenList
 from typing import Iterable, TypeVar, Any, Dict, Optional, List, cast
 
 from xdsl.dialects.memref import AnyUnrankedMemrefType, MemRefType, UnrankedMemrefType
@@ -12,11 +11,11 @@ from xdsl.ir import (BlockArgument, MLIRType, SSAValue, Block, Callable,
 from xdsl.utils.diagnostic import Diagnostic
 from xdsl.dialects.builtin import (
     AnyIntegerAttr, AnyFloatAttr, AnyUnrankedTensorType, AnyVectorType,
-    DenseIntOrFPElementsAttr, Float16Type, Float32Type, Float64Type, FloatAttr,
-    IndexType, IntegerType, NoneAttr, OpaqueAttr, Signedness, StringAttr,
-    FlatSymbolRefAttr, IntegerAttr, ArrayAttr, IntAttr, TensorType, UnitAttr,
-    FunctionType, UnrankedTensorType, UnregisteredOp, VectorType,
-    DictionaryAttr)
+    DenseIntOrFPElementsAttr, DenseResourceAttr, Float16Type, Float32Type,
+    Float64Type, FloatAttr, IndexType, IntegerType, NoneAttr, OpaqueAttr,
+    Signedness, StringAttr, FlatSymbolRefAttr, IntegerAttr, ArrayAttr, IntAttr,
+    TensorType, UnitAttr, FunctionType, UnrankedTensorType, UnregisteredOp,
+    VectorType, DictionaryAttr)
 
 indentNumSpaces = 2
 
@@ -59,7 +58,7 @@ class Printer:
                 self.print_region(arg)
                 continue
             if isinstance(arg, Block):
-                self.print_block_name(arg)
+                self.print_block(arg)
                 continue
             if isinstance(arg, Operation):
                 self.print_op(arg)
@@ -226,7 +225,10 @@ class Printer:
             self._block_names[block] = self._get_new_valid_block_id()
         self.print(self._block_names[block])
 
-    def _print_named_block(self, block: Block) -> None:
+    def print_block(self, block: Block) -> None:
+        if not isinstance(block, Block):
+            raise TypeError('Expected a Block; got %s' % type(block).__name__)
+
         self.print_block_name(block)
         if len(block.args) > 0:
             self.print("(")
@@ -246,6 +248,9 @@ class Printer:
         self.print_attribute(arg.typ)
 
     def print_region(self, region: Region) -> None:
+        if not isinstance(region, Region):
+            raise TypeError('Expected a Region; got %s' %
+                            type(region).__name__)
         if len(region.blocks) == 0:
             self.print("{}")
             return
@@ -259,7 +264,7 @@ class Printer:
         self.print("{")
         self._print_new_line()
         for block in region.blocks:
-            self._print_named_block(block)
+            self.print_block(block)
         self.print("}")
 
     def print_regions(self, regions: List[Region]) -> None:
@@ -274,7 +279,7 @@ class Printer:
             self.print(" ")
             self.print_list(regions, self.print_region, delimiter=" ")
 
-    def _print_operands(self, operands: FrozenList[SSAValue]) -> None:
+    def _print_operands(self, operands: tuple[SSAValue, ...]) -> None:
         if len(operands) == 0:
             self.print("()")
             return
@@ -427,6 +432,13 @@ class Printer:
             self.print(attribute.type)
             return
 
+        # Dense resources have an alias in MLIR, but not in xDSL
+        if isinstance(attribute, DenseResourceAttr):
+            handle = attribute.resource_handle.data
+            type = attribute.type
+            self.print(f"dense_resource<{handle}> : ", type)
+            return
+
         # vector types have an alias in MLIR, but not in xDSL
         if ((isinstance(attribute, VectorType)
              or isinstance(attribute, TensorType))
@@ -586,6 +598,9 @@ class Printer:
                 self.print(")")
 
     def _print_op(self, op: Operation) -> None:
+        if not isinstance(op, Operation):
+            raise TypeError('Expected an Operation; got %s' %
+                            type(op).__name__)
         begin_op_pos = self._current_column
         self._print_results(op)
         if isinstance(op, UnregisteredOp):
