@@ -27,9 +27,10 @@ class Diagnostic:
         if isinstance(toplevel, Operation):
             p.print_op(toplevel)
         elif isinstance(toplevel, Block):
-            p._print_named_block(toplevel)  # type: ignore
+            p.print_block(toplevel)
         elif isinstance(toplevel, Region):
-            p._print_region(toplevel)  # type: ignore
+            # TOFIX: Is that ever used. Revisit the whole exception
+            p._print_region(toplevel)  # TOFIX #type: ignore
         else:
             assert "xDSL internal error: get_toplevel_object returned unknown construct"
 
